// PriorityQueue provides an implementation of the Queue interface that uses a
// LinkedList to store the elements of the queue.  All operations will execute
// in O(1) time.  This particular queue is a priority queue.  It assumes that
// all values inserted into the queue implement the Comparable interface and
// are mututally comparable.  Values are dequeued in priority order ("less"
// means it is dequeued earlier) and otherwise dequeued in FIFO order (when
// the priorities are equal).

import java.util.*;

public class PriorityQueue implements Queue {
    private LinkedList elements;

    // post: constructs an empty queue
    public PriorityQueue() {
        elements = new LinkedList();
    }

    // post: given value inserted in the middle of the queue first by priority
    //       and then by queue order (FIFO)
    public void enqueue(Object v) {
        Comparable value = (Comparable)v;
        if (elements.isEmpty() || value.compareTo(elements.getFirst()) < 0)
            elements.addFirst(value);
        else {
            ListIterator i = elements.listIterator(elements.size());
            while (i.hasPrevious())
                if (value.compareTo(i.previous()) >= 0) {
                    i.next();
                    i.add(value);
                    break;
                }
        }
    }
                    
    // pre : !isEmpty()
    // post: removes and returns the value at the front of the queue
    public Object dequeue() {
        if (isEmpty())
            throw new IllegalStateException("attempt to dequeue empty queue");
        return elements.removeFirst();
    }

    // post: returns true if the queue is empty, false otherwise
    public boolean isEmpty() {
        return elements.isEmpty();
    }

    // post: returns the current number of element in the queue
    public int size() {
        return elements.size();
    }

    // post: returns a String representation of the queue contents
    public String toString() {
        return "front " + elements + " back";
    }
}
